using System;
using System.Collections;
using Server.Misc;
using Server.Mobiles;
using Server.Network;
using Server.Gumps;
using Server.Regions;

namespace Server.Items
{
	[DispellableFieldAttribute]
	public class Moongate : Item
	{
		private Point3D m_Target;
		private Map m_TargetMap;
		private bool m_bDispellable;

		[CommandProperty( AccessLevel.GameMaster )]
		public Point3D Target
		{
			get
			{
				return m_Target;
			}
			set
			{
				m_Target = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Map TargetMap
		{
			get
			{
				return m_TargetMap;
			}
			set
			{
				m_TargetMap = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public bool Dispellable
		{
			get
			{
				return m_bDispellable;
			}
			set
			{
				m_bDispellable = value;
			}
		}


		[Constructable]
		public Moongate() : this( Point3D.Zero, null )
		{
			m_bDispellable = true;
		}

		[Constructable]
		public Moongate(bool bDispellable) : this( Point3D.Zero, null )
		{
			m_bDispellable = bDispellable;
		}

		public Moongate( Point3D target, Map targetMap ) : base( 0xF6C )
		{
			Movable = false;
			Light = LightType.Circle300;

			m_Target = target;
			m_TargetMap = targetMap;
		}

		public Moongate( Serial serial ) : base( serial )
		{
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( !from.Player )
				return;

			if ( from.InRange( GetWorldLocation(), 1 ) )
				CheckGate( from, 1 );
			else
				from.SendLocalizedMessage( 500446 ); // That is too far away.
		}

		public override bool OnMoveOver( Mobile m )
		{
			if ( m.Player )
				CheckGate( m, 0 );

			return true;
		}

		public virtual void CheckGate( Mobile m, int range )
		{
			new DelayTimer( m, this, range ).Start();
		}

		public virtual void UseGate( Mobile m )
		{
			if ( m_TargetMap != null )
			{
				BaseCreature.TeleportPets( m, m_Target, m_TargetMap );

				m.Map = m_TargetMap;
				m.Location = m_Target;

				m.PlaySound( 0x1FE );
			}
			else
			{
				m.SendMessage( "This moongate does not seem to go anywhere." );
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 1 ); // version

			writer.Write( m_Target );
			writer.Write( m_TargetMap );
			
			// Version 1
			writer.Write( m_bDispellable );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			m_Target = reader.ReadPoint3D();
			m_TargetMap = reader.ReadMap();

			if (version >= 1)
			{
				m_bDispellable = reader.ReadBool();
			}
		}

		public static bool IsInTown( Point3D p, Map map )
		{
			if ( map == null )
				return false;

			Region r = Region.Find( p, map );

			return ( r is TrammelTown || r is FeluccaTown );
		}

		private class DelayTimer : Timer
		{
			private Mobile m_From;
			private Moongate m_Gate;
			private int m_Range;

			public DelayTimer( Mobile from, Moongate gate, int range ) : base( TimeSpan.FromSeconds( 1.0 ) )
			{
				m_From = from;
				m_Gate = gate;
				m_Range = range;
			}

			protected override void OnTick()
			{
				if ( m_From.Deleted || m_Gate.Deleted || m_From.Map != m_Gate.Map || !m_From.InRange( m_Gate, m_Range ) )
					return;

				if ( m_Gate.m_TargetMap != null )
				{
					if ( IsInTown( m_From.Location, m_From.Map ) && !IsInTown( m_Gate.m_Target, m_Gate.m_TargetMap ) )
					{
						m_From.Send( new PlaySound( 0x20E, m_From.Location ) );
						m_From.CloseGump( typeof( MoongateConfirmGump ) );
						m_From.SendGump( new MoongateConfirmGump( m_From, m_Gate ) );
					}
					else
					{
						m_Gate.UseGate( m_From );
					}
				}
				else
				{
					m_From.SendMessage( "This moongate does not seem to go anywhere." );
				}
			}
		}
	}

	public class MoongateConfirmGump : Gump
	{
		private Mobile m_From;
		private Moongate m_Gate;

		public MoongateConfirmGump( Mobile from, Moongate gate ) : base( 20, 30 )
		{
			m_From = from;
			m_Gate = gate;

			AddPage( 0 );

			AddBackground( 0, 0, 420, 400, 5054 );
			AddBackground( 10, 10, 400, 380, 3000 );

			AddHtml( 20, 40, 380, 60, @"Dost thou wish to step into the moongate? Continue to enter the gate, Cancel to stay here", false, false );

			AddHtmlLocalized( 55, 110, 290, 20, 1011012, false, false ); // CANCEL
			AddButton( 20, 110, 4005, 4007, 0, GumpButtonType.Reply, 0 );

			AddHtmlLocalized( 55, 140, 290, 40, 1011011, false, false ); // CONTINUE
			AddButton( 20, 140, 4005, 4007, 1, GumpButtonType.Reply, 0 );
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			if ( m_From.Deleted || m_Gate.Deleted || m_From.Map != m_Gate.Map )
				return;

			if ( info.ButtonID == 1 )
			{
				if ( m_From.InRange( m_Gate.GetWorldLocation(), 1 ) )
					m_Gate.UseGate( m_From );
				else
					m_From.SendLocalizedMessage( 500446 ); // That is too far away.
			}
		}
	}
}